package org.mapsforge.android.maps;

import java.util.ArrayList;
import java.util.Collection;

import android.app.AlertDialog;
import android.content.Context;
import android.graphics.drawable.Drawable;

/**
 * Thread-safe implementation of the {@link ItemizedOverlay} class using an
 * {@link ArrayList} as internal data structure.
 */
public class ArrayItemizedOverlayTweaked extends ItemizedOverlay<OverlayItem> {
	private static final int ARRAY_LIST_INITIAL_CAPACITY = 8;
	private static final String THREAD_NAME = "ArrayItemizedOverlayTweaked";

	protected final Context context;
	protected AlertDialog.Builder dialog;
	protected OverlayItem item;
	protected final ArrayList<OverlayItem> overlayItems;

	/**
	 * Constructs a new ArrayItemizedOverlay.
	 * 
	 * @param defaultMarker
	 *            the default marker.
	 * @param context
	 *            the reference to the application context.
	 */
	public ArrayItemizedOverlayTweaked(Drawable defaultMarker, Context context) {
		super(boundCenterBottom(defaultMarker));
		this.context = context;
		this.overlayItems = new ArrayList<OverlayItem>(
				ARRAY_LIST_INITIAL_CAPACITY);
	}

	/**
	 * Adds the given item to the Overlay.
	 * 
	 * @param overlayItem
	 *            the item that should be added to the Overlay.
	 */
	public synchronized void addOverlay(OverlayItem overlayItem) {
		this.overlayItems.add(overlayItem);
		populate();
	}

	/**
	 * Adds all items of the given collection to the Overlay.
	 * 
	 * @param c
	 *            collection whose items should be added to the Overlay.
	 */
	public synchronized void addOverlays(Collection<? extends OverlayItem> c) {
		this.overlayItems.addAll(c);
		populate();
	}

	/**
	 * Removes all items from the Overlay.
	 */
	public synchronized void clear() {
		this.overlayItems.clear();
		populate();
	}

	@Override
	public String getThreadName() {
		return THREAD_NAME;
	}

	/**
	 * Removes the given item from the Overlay.
	 * 
	 * @param overlayItem
	 *            the item that should be removed from the Overlay.
	 */
	public synchronized void removeOverlay(OverlayItem overlayItem) {
		this.overlayItems.remove(overlayItem);
		populate();
		super.requestRedraw();
	}

	@Override
	public synchronized int size() {
		return this.overlayItems.size();
	}

	@Override
	public synchronized OverlayItem createItem(int i) {
		return this.overlayItems.get(i);
	}

	@Override
	// you always need this even it doesn't do anything visibly
	protected synchronized boolean onTap(int index) {

		OverlayItem item = createItem(index);

		if (item != null)
			System.out.println("ArrayItemizedOverlayTweaked ontap");
		return true;
		// this.dialog = new AlertDialog.Builder(this.context);
		// this.dialog.setTitle(this.item.getTitle());
		// this.dialog.setMessage(this.item.getSnippet());
		// this.dialog.show();
		//
		// return false;
	}

}
