package org.tipple.mapDisplay;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Locale;

import org.mapsforge.android.maps.MapActivity;
import org.mapsforge.android.maps.MapView;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.AssetManager;
import android.graphics.drawable.Drawable;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.TextView;

public class tippleActivity extends MapActivity implements OnInitListener {// OnSharedPreferenceChangeListener
	enum TextAudioModeEnum {
		TEXT_ONLY, TEXT_PLUS_AUDIO, AUDIO_ONLY, AUDIO_PLUS_TEXT
	};

	enum bookModeEnum {
		BOOK_A, BOOK_B, BOOK_AB
	};

	// image loader
	public ImageLoader loader;
	public static TippleLog log;
	public static bookModeEnum current_book_mode;
	protected tippleActivity.bookModeEnum current_book_mode_;
	// File storage to run on the emulator initially
	public static String geodataDirectory = "/sdcard/";
	public static String logDirectory = "/sdcard/";
	public static String audioDirectory = "/sdcard/";
	public static String imageDirectory = "/sdcard/";
	// File storage to run on the real device
	// public static String geodataDirectory;
	// public static String logDirectory;
	// public static String audioDirectory;

	// Views
	protected TextView longlat_view_;
	protected MapView mapview;
	protected TextView text_view_;
	protected FrameLayout text_map_composite_;

	protected ArrayList<tippleLocation> locations = null;
	protected ArrayList<tippleLocation> user_trail_ = null;
	protected String user_trail_filename_ = null;
	protected TippleLocationManagerVersion2 tip_location_manager_,
			my_location_manager;
	// Preferences
	public static SharedPreferences sharedPreferences;
	public static boolean centreOnGPSLocation;
	public static boolean showScaleBar;
	public static boolean showSights;
	public static boolean showUserTrail;
	public static TextAudioModeEnum textAudioMode;
	public static bookModeEnum bookMode;
	public static boolean bookA;
	public static boolean bookB;
	public static String bookModeStr;
	public static Boolean currentShowSights = false;
	public static Boolean currentShowUserTrail = false;

	// Text to Speech
	protected TextToSpeech tts_ = null;
	// Request codes
	protected final int TTS_DATA_CHECK = 0;
	private static final int SELECT_LOG_FILE = 1;

	protected LocationListener mylocationListener = null;
	public static Drawable highlightMarker1;
	public static Drawable highlightMarker2;

	// Text to Speech
	private final String MY_DEBUG_TAG = "DL xml data retreive";
	private static final long MINIMUM_DISTANCE_CHANGE_FOR_UPDATES = 1; // in
																		// Meters
	private static final long MINIMUM_TIME_BETWEEN_UPDATES = 1000; // in
																	// miniseconds
	protected AudioPlayerView audio_player_view_;
	protected LocationManager locationManager;
	protected Button retrieveLocationButton;
	protected double currentLongitude;
	protected double currentLantitude;
	protected double currentNodeId;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		// Set up logging
		log = new TippleLog(this, logDirectory);
		log.optStartLog();

		refreshPreferences();
		SharedPreferences.Editor editor = sharedPreferences.edit();
		editor.putBoolean("showUserTrail", false);

		// Set up Text to Speech
		Intent checkIntent = new Intent();
		checkIntent.setAction(TextToSpeech.Engine.ACTION_CHECK_TTS_DATA);
		startActivityForResult(checkIntent, TTS_DATA_CHECK);

		mapview = createMapView("/sdcard/hamilton.map");
		// For running on the real phone,change all file to tipple-store
		// and asset folder
		// mapview = createMapViewFromAsset("hamilton.map");

		longlat_view_ = createLongLatView();
		LinearLayout linearLayout = new LinearLayout(this);
		linearLayout.setOrientation(LinearLayout.VERTICAL);
		text_view_ = createTextView();
		text_map_composite_ = compositTextViewOnMapView(mapview, text_view_);
		audio_player_view_ = createAudioPlayerView();

		linearLayout.addView(longlat_view_);
		linearLayout.addView(text_map_composite_);
		linearLayout.addView(audio_player_view_);
		setContentView(linearLayout);

		// load image in background and store in sdcard
		load(this);
	}

	protected void load(Activity activity) {

		try {
			for (int i = 0; i < ImageURL.URLS.length; i++) {
				loader = new ImageLoader(activity);
				loader.download(ImageURL.URLS[i]);
			}
		} catch (Exception ex) {
			System.out.println("tippleActivity load... error loading");
		}

	}

	protected boolean assetToExternalStorage(String asset_filename,
			String external_storage_filename) {
		boolean status = true;

		try {

			File external_storage_file = new File(external_storage_filename);
			if (!external_storage_file.exists()) {
				// Need to create the /sdcard version

				AssetManager assetManager = getAssets();

				InputStream ais = assetManager.open(asset_filename); // ais =
																		// asset
																		// input
																		// stream
				FileOutputStream fos = new FileOutputStream(
						external_storage_file); // fos = file output stream

				byte[] buffer = new byte[1024];

				int buflen;
				while ((buflen = ais.read(buffer)) > 0) {
					fos.write(buffer, 0, buflen);
				}

				ais.close();
				fos.close();
			}

		} catch (Exception e) {
			e.printStackTrace();
			status = false;
		}

		return status;
	}

	protected boolean initTippleStore(String location) {

		// Locate map data directory
		File external_root_file = Environment.getExternalStorageDirectory();
		String external_root = external_root_file.getAbsolutePath();
		String tipple_store_dir = external_root + "/tipple-store";

		geodataDirectory = tipple_store_dir + "/geodata/";
		// logDirectory = tipple_store_dir + "/logs/";
		audioDirectory = tipple_store_dir + "/audio/";

		File tipple_store_file = new File(tipple_store_dir);
		if (!tipple_store_file.exists()) {
			if (!tipple_store_file.mkdir()) {
				System.err.println("Failed to create directory: "
						+ tipple_store_dir);
				return false;
			}
		}

		File geodataDirFile = new File(geodataDirectory);
		if (!geodataDirFile.exists()) {
			if (!geodataDirFile.mkdir()) {
				System.err.println("Failed to create directory: "
						+ geodataDirectory);
				return false;
			}
		}

		File audioDirFile = new File(audioDirectory);
		if (!audioDirFile.exists()) {
			if (!audioDirFile.mkdir()) {
				System.err.println("Failed to create directory: "
						+ audioDirectory);
				return false;
			}
		}

		// dig out the audio files
		String[] audio_filenames = { "BeepMorseSonar.wav", "Ding.wav",
				"DingExtended.wav" };
		for (String audio_filename : audio_filenames) {
			String audio_asset_filename = "audio/" + audio_filename;
			String audio_exstore_filename = audioDirectory + audio_filename;

			if (!assetToExternalStorage(audio_asset_filename,
					audio_exstore_filename)) {
				System.err.println("Failed to transfer asset "
						+ audio_asset_filename + " to "
						+ audio_exstore_filename);
				return false;
			}
		}

		if ((location != null) && (!location.equals(""))) {
			// get out map and XML location file

			String map_asset_filename = "geodata/" + location + ".map";
			String map_exstore_filename = geodataDirectory + location + ".map";

			if (!assetToExternalStorage(map_asset_filename,
					map_exstore_filename)) {
				System.err.println("Failed to transfer asset "
						+ map_asset_filename + " to " + map_exstore_filename);
				return false;
			}

		}

		return true;
	}

	protected boolean initTippleStore() {
		return initTippleStore(null);
	}

	protected FrameLayout compositTextViewOnMapView(MapView map_view,
			TextView text_view) {
		FrameLayout frame_layout = new FrameLayout(this);

		LayoutParams view_layout = new LinearLayout.LayoutParams(
				ViewGroup.LayoutParams.WRAP_CONTENT,
				ViewGroup.LayoutParams.WRAP_CONTENT, 1);

		frame_layout.setLayoutParams(view_layout);
		frame_layout.addView(map_view);
		frame_layout.addView(text_view);

		return frame_layout;
	}

	protected TextView createLongLatView() {
		// Lat,Long line at top
		TextView longlat_view = new TextView(this);
		longlat_view.setText("(Long,Lat): ");

		LayoutParams longlat_view_layout = new LinearLayout.LayoutParams(
				ViewGroup.LayoutParams.MATCH_PARENT, 35, 0);
		longlat_view.setLayoutParams(longlat_view_layout);

		return longlat_view;
	}

	protected TextView createTextView() {
		// Text view in middle (ultimately on top of map view)
		TextView text_view = new TextView(this);

		LayoutParams text_view_layout = new LinearLayout.LayoutParams(
				ViewGroup.LayoutParams.MATCH_PARENT,
				ViewGroup.LayoutParams.WRAP_CONTENT, 1);
		text_view.setLayoutParams(text_view_layout);

		text_view.setGravity(Gravity.BOTTOM | Gravity.FILL_HORIZONTAL);

		text_view.setPadding(12, 12, 12, 12);
		text_view.setTextColor(0xffffffff);
		text_view.setBackgroundColor(0xCC000000);
		text_view.setTextSize(20.0f);

		text_view.setVerticalScrollBarEnabled(true);
		text_view.setScrollBarStyle(View.SCROLLBARS_INSIDE_INSET);

		text_view.setMovementMethod(new ScrollingMovementMethod());

		text_view.setVisibility(View.INVISIBLE);

		return text_view;
	}

	protected MapView createMapView(String map_filename) {
		// Map view in middle
		MapView mapview = new MapView(this);
		mapview.setClickable(true);
		mapview.setBuiltInZoomControls(true);
		// String full_map_filename = geodataDirectory + map_filename;
		// mapview.setMapFile(full_map_filename);
		mapview.setMapFile(map_filename);
		LayoutParams map_view_layout = new LinearLayout.LayoutParams(
				ViewGroup.LayoutParams.WRAP_CONTENT,
				ViewGroup.LayoutParams.WRAP_CONTENT, 1);
		mapview.setLayoutParams(map_view_layout);
		return mapview;

	}

	protected MapView createMapViewFromAsset(String map_filename) {
		// Transfer asset to file in dataDirectory if it does not already exist
		String asset_filename = "geodata" + File.separator + map_filename;
		String external_storage_filename = geodataDirectory + map_filename;

		assetToExternalStorage(asset_filename, external_storage_filename);

		return createMapView(map_filename);

	}

	protected AudioPlayerView createAudioPlayerView() {
		// Player at the bottom
		AudioPlayerView audio_player_view = new AudioPlayerView(this,
				text_view_);

		LayoutParams audio_player_layout = new LinearLayout.LayoutParams(
				ViewGroup.LayoutParams.MATCH_PARENT, 100, 0);
		audio_player_view.setLayoutParams(audio_player_layout);

		return audio_player_view;
	}

	protected void refreshPreferences() {
		// Restore preferences
		sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

		showScaleBar = sharedPreferences.getBoolean("showScaleBar", false);
		centreOnGPSLocation = sharedPreferences.getBoolean("centreOnGPSLoc",
				false);

		showSights = sharedPreferences.getBoolean("showSights", true);
		showUserTrail = sharedPreferences.getBoolean("showUserTrail", false);

		String textAudioModeStr = sharedPreferences.getString("textAudioMode",
				"AUDIO_PLUS_TEXT");
		textAudioMode = Enum.valueOf(TextAudioModeEnum.class, textAudioModeStr);

		// change book selection from radio button to check box
		bookA = sharedPreferences.getBoolean("showBookA", false);
		bookB = sharedPreferences.getBoolean("showBookB", false);

		if (bookA) {
			if (!bookB)
				bookModeStr = "BOOK_A";
			else
				bookModeStr = "BOOK_AB";
		} else {
			if (!bookB)
				bookModeStr = "BOOK_B";// by default, show one books
			else
				bookModeStr = "BOOK_B";

		}

		bookMode = Enum.valueOf(bookModeEnum.class, bookModeStr);
	}

	protected void displayLocationOnMap() {
		locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		tip_location_manager_ = new TippleLocationManagerVersion2(this,
				mapview, longlat_view_, locationManager, audio_player_view_,
				tts_);

	}

	Boolean isActive = true;
	Boolean showPath = true;

	@Override
	protected void onResume() {
		System.err.println("*** TippleActivity::onResume()");
		super.onResume();
		refreshPreferences();
		audio_player_view_.refreshView();// try this way can we get locations
		mapview.setScaleBar(showScaleBar);
		if (tip_location_manager_ != null) {
			refreshMapView();

		}

	}

	@Override
	protected void onStop() {
		System.err.println("*** TippleActivity::onStop()");
		super.onStop();

		SharedPreferences.Editor editor = sharedPreferences.edit();
		editor.commit();

	}

	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		System.err
				.println("*** TippleActivity::onActivityResult(), requestCode: "
						+ requestCode);

		if (requestCode == TTS_DATA_CHECK) {
			if (resultCode == TextToSpeech.Engine.CHECK_VOICE_DATA_PASS) {
				// success, create the TTS instance
				tts_ = new TextToSpeech(this,
						(TextToSpeech.OnInitListener) this);
				tts_.setLanguage(Locale.UK);
				displayLocationOnMap();
				load(this);

			} else {
				// missing data, install it
				Intent installIntent = new Intent();
				installIntent
						.setAction(TextToSpeech.Engine.ACTION_INSTALL_TTS_DATA);
				startActivity(installIntent);
			}
		}

		else if (requestCode == SELECT_LOG_FILE) {

		}

	}

	public void onInit(int status) {
		System.err
				.println("*** TippleActivity::onInit(), status (zero == good): "
						+ status);

		if (status == TextToSpeech.SUCCESS) {
			Log.d("TippleActivity::onInit()",
					"Text-to-speech engine initialized");
		} else if (status == TextToSpeech.ERROR) {
			Log.e("TippleActivity::onInit()",
					"Text-to-speech engine failed initialized");
			System.err
					.println("Error occured while initializing text-to-speech engine");
		}

	}

	protected void onDestroy() {
		System.err.println("*** TippleActivity::onDestroy()");
		super.onDestroy();

		tts_.shutdown();
		tts_ = null;

		// Switch off location updates
		if (locationManager != null) {

			if (mylocationListener != null) {

				locationManager.removeUpdates(mylocationListener);
				mylocationListener = null;
			}
			mylocationListener = null;

		}

		tip_location_manager_.stopOverlayThreads();

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.options_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {

		case R.id.menu_preferences:
			startActivity(new Intent(this, EditPreferences.class));
			return true;

		default:
			return false;
		}
	}

	// @Override
	public void refreshMapView() {

		if (current_book_mode_ != tippleActivity.bookMode) {
			// book mode has beeen changed
			tip_location_manager_.resetOverlayThreads();// reset overlay item
			System.out.println("Clear all arraylist");
			if (tippleActivity.bookMode == tippleActivity.bookModeEnum.BOOK_A) {

				locations = tip_location_manager_.loadTour(true);

			}

			else if (tippleActivity.bookMode == tippleActivity.bookModeEnum.BOOK_B) {
				locations = tip_location_manager_.loadTour(false);

			} else if (tippleActivity.bookMode == tippleActivity.bookModeEnum.BOOK_AB) {
				locations = tip_location_manager_.loadTourDescription();

			}
			current_book_mode_ = bookMode;
			displayLocationOnMapDynamically(locations);

		}
		showTour();
		System.out.println("refreshMapView when mode not changed ");
	}

	protected void displayLocationOnMapDynamically(
			ArrayList<tippleLocation> locations) {

		if (mylocationListener != null) {
			((locationListener) mylocationListener)
					.resetOverlayThreadsOnListener();
			locationManager.removeUpdates(mylocationListener);

		}
		mylocationListener = null;
		System.gc();
		audio_player_view_.resetAudioView();
		audio_player_view_.refreshView();

		tip_location_manager_.createTourLocationsOverlay(mapview, locations);
		mylocationListener = new locationListener(this, mapview, longlat_view_,
				locationManager, tip_location_manager_, locations);
		locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
				MINIMUM_TIME_BETWEEN_UPDATES,
				MINIMUM_DISTANCE_CHANGE_FOR_UPDATES, mylocationListener);
		System.out.println("displayLocationOnmapDy created listener");

		if (showSights) {
			if (showUserTrail)
				tip_location_manager_.addTourLocations(mapview, true);
			else {
				tip_location_manager_.resetRouteOverlay();
				tip_location_manager_.addTourLocations(mapview, false);
			}
		} else {
			tip_location_manager_.removeTourLocations(mapview);
		}

	}

	/*
	 * when showSignts or showUserTrail has been changed will call this method
	 */
	protected void showTour() {
		if ((currentShowSights != showSights)
				|| (currentShowUserTrail != showUserTrail)) {
			if (showSights) {
				if (showUserTrail)
					tip_location_manager_.addTourLocations(mapview, true);
				else {
					tip_location_manager_.resetRouteOverlay();
					tip_location_manager_.addTourLocations(mapview, false);
				}
			} else {
				tip_location_manager_.removeTourLocations(mapview);
			}
			currentShowSights = showSights;
			currentShowUserTrail = showUserTrail;
		}
	}

}